using System;
using System.Collections.Generic;
using System.Linq;
using Labs.Interfaces;

namespace Labs.Business
{
    public class CustomerManager
    {
        public CustomerManager(ICustomerRepository instance)
        {
            if (instance == null)
                throw new ArgumentNullException("instance",
                    "instance is null.");

            m_RepositoryInstance = instance;
        }

        public string CreateAccountCode(string customerName)
        {
            if (String.IsNullOrEmpty(customerName) == true)
                throw new ArgumentException(
                    "customerName is null or empty.", "customerName");

            bool isValid = false;

            string suggestedAccountCode = null;

            while (isValid == false)
            {
                suggestedAccountCode = GetSuggestedAccountCode();

                if (RepositoryInstance.IsAccountCodeInUse(
                    suggestedAccountCode) == false)
                {
                    isValid = true;
                }
            }

            if (isValid == true)
            {
                RepositoryInstance.SaveCustomer(
                    suggestedAccountCode, customerName);
                return suggestedAccountCode;
            }
            else
            {
                throw new InvalidOperationException("Duplicate account code.");
            }
        }

        public string GetSuggestedAccountCode()
        {
            string guid = Guid.NewGuid().ToString();

            return guid.Substring(0, 6);
        }

        private ICustomerRepository m_RepositoryInstance;
        private ICustomerRepository RepositoryInstance
        {
            get { return m_RepositoryInstance; }
        }
    }
}
