using System;
using System.Collections.Generic;
using System.Linq;
using Labs.Interfaces;

namespace Labs.DataAccess
{
    public class SqlPersonRepository<T> : IPersonRepository
        where T : IPerson, new()
    {
        public IList<IPerson> GetAll()
        {
            var tableAdapter =
                new PersonDataSetTableAdapters.PersonTableAdapter();

            var results = tableAdapter.GetData();

            if (results == null || results.Count == 0)
            {
                return null;
            }
            else
            {
                var adapter = new PersonToDataSetAdapter();

                var toValues = new List<IPerson>();

                adapter.Adapt<T>(results, toValues);

                return toValues;
            }
        }

        public IPerson GetById(int id)
        {
            var tableAdapter =
                new PersonDataSetTableAdapters.PersonTableAdapter();

            var results = tableAdapter.GetDataById(id);

            if (results == null || results.Count == 0)
            {
                return null;
            }
            else
            {
                var adapter = new PersonToDataSetAdapter();

                IPerson toValue = new T();

                adapter.Adapt(results[0], toValue);

                return toValue;
            }
        }

        public void Save(IPerson saveThis)
        {
            if (saveThis == null)
                throw new ArgumentNullException("saveThis", "saveThis is null.");

            if (saveThis.Id == 0)
            {
                Insert(saveThis);
            }
            else
            {
                Update(saveThis);
            }
        }

        private void Update(IPerson saveThis)
        {
            var tableAdapter =
                new PersonDataSetTableAdapters.PersonTableAdapter();

            var dataset = new PersonDataSet();

            tableAdapter.FillById(dataset.Person, saveThis.Id);

            if (dataset.Person == null || dataset.Person.Count == 0)
            {
                throw new InvalidOperationException("Unknown person.");
            }
            else
            {
                var adapter = new PersonToDataSetAdapter();

                adapter.Adapt(saveThis, dataset.Person[0], true);

                Save(dataset);
            }
        }

        private void Insert(IPerson saveThis)
        {
            var dataset = new PersonDataSet();

            var toValue = dataset.Person.NewPersonRow();

            var adapter = new PersonToDataSetAdapter();

            adapter.Adapt(saveThis, toValue);

            dataset.Person.AddPersonRow(toValue);

            Save(dataset);

            adapter.Adapt(toValue, saveThis);
        }

        private void Save(PersonDataSet dataset)
        {
            var tableAdapter =
                new PersonDataSetTableAdapters.PersonTableAdapter();

            tableAdapter.Update(dataset.Person);
        }
    }
}
