﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using DataAccessTesting.Interfaces;
using DataAccessTesting.Business;
using DataAccessTesting.DataAccess;

namespace DataAccessTesting.Tests
{
    [TestClass]
    public class PersonToDataSetAdapterFixture
    {
        [TestMethod]
        public void AdaptFromPersonToDataRow()
        {
            IPerson fromValue = new Person();

            fromValue.Id = 1234;
            fromValue.FirstName = "Skip";
            fromValue.LastName = "Rosenwinkle";
            fromValue.EmailAddress = "skipr@yahoo.net";

            var toValue = new PersonDataSet().Person.NewPersonRow();

            var adapter = new PersonToDataSetAdapter();

            adapter.Adapt(fromValue, toValue);

            AssertAreEqual(fromValue, toValue);
        }

        private void AssertAreEqual(
            IPerson expected, PersonDataSet.PersonRow actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<string>(expected.FirstName,
                actual.FirstName, "FirstName");
            Assert.AreEqual<string>(expected.LastName,
                actual.LastName, "LastName");
            Assert.AreEqual<string>(expected.EmailAddress,
                actual.EmailAddress, "EmailAddress");
        }

        [TestMethod]
        public void AdaptFromDataRowToPerson()
        {
            var fromValue = new PersonDataSet().Person.NewPersonRow();

            fromValue.Id = 1234;
            fromValue.FirstName = "Skip";
            fromValue.LastName = "Rosenwinkle";
            fromValue.EmailAddress = "skipr@yahoo.net";

            IPerson toValue = new Person();

            var adapter = new PersonToDataSetAdapter();

            adapter.Adapt(fromValue, toValue);

            AssertAreEqual(fromValue, toValue);
        }

        private void AssertAreEqual(
            PersonDataSet.PersonRow expected, IPerson actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<string>(expected.FirstName,
                actual.FirstName, "FirstName");
            Assert.AreEqual<string>(expected.LastName,
                actual.LastName, "LastName");
            Assert.AreEqual<string>(expected.EmailAddress,
                actual.EmailAddress, "EmailAddress");
        }
    }
}
