﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using DataAccessTesting.Interfaces;
using DataAccessTesting.Business;
using DataAccessTesting.DataAccess;

namespace DataAccessTesting.Tests
{
    [TestClass]
    public class PersonToDataSetAdapterFixture
    {
        [TestMethod]
        public void AdaptFromPersonToDataRow()
        {
            IPerson fromValue = new Person();

            fromValue.Id = 1234;
            fromValue.FirstName = "Skip";
            fromValue.LastName = "Rosenwinkle";
            fromValue.EmailAddress = "skipr@yahoo.net";

            var toValue = new PersonDataSet().Person.NewPersonRow();

            var adapter = new PersonToDataSetAdapter();

            adapter.Adapt(fromValue, toValue);

            AssertAreEqual(fromValue, toValue);
        }

        private void AssertAreEqual(
            IPerson expected, PersonDataSet.PersonRow actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<string>(expected.FirstName,
                actual.FirstName, "FirstName");
            Assert.AreEqual<string>(expected.LastName,
                actual.LastName, "LastName");
            Assert.AreEqual<string>(expected.EmailAddress,
                actual.EmailAddress, "EmailAddress");
        }

        [TestMethod]
        public void AdaptFromDataRowToPerson()
        {
            var fromValue = new PersonDataSet().Person.NewPersonRow();

            fromValue.Id = 1234;
            fromValue.FirstName = "Skip";
            fromValue.LastName = "Rosenwinkle";
            fromValue.EmailAddress = "skipr@yahoo.net";

            IPerson toValue = new Person();

            var adapter = new PersonToDataSetAdapter();

            adapter.Adapt(fromValue, toValue);

            AssertAreEqual(fromValue, toValue);
        }

        [TestMethod]
        public void AdaptFromDataTableToListOfPerson()
        {
            var fromValues = new PersonDataSet().Person;
            IList<IPerson> toValues = new List<IPerson>();

            int expectedNumberOfRecords = 3;

            CreateSampleValues(fromValues, expectedNumberOfRecords);

            Assert.AreEqual<int>(expectedNumberOfRecords, fromValues.Count, "Test record count was wrong.");

            var adapter = new PersonToDataSetAdapter();

            adapter.Adapt<Person>(fromValues, toValues);

            AssertAreEqual(fromValues, toValues);
        }

        private void AssertAreEqual(PersonDataSet.PersonDataTable fromValues, IList<IPerson> toValues)
        {
            if (fromValues == null)
                throw new ArgumentNullException("fromValues", "fromValues is null.");
            if (toValues == null)
                throw new ArgumentNullException("toValues", "toValues is null.");

            Assert.AreEqual<int>(fromValues.Count, toValues.Count, "Record count did not match.");

            for (int i = 0; i < fromValues.Count; i++)
            {
                AssertAreEqual(fromValues[i], toValues[i]);
            }
        }

        private void CreateSampleValues(PersonDataSet.PersonDataTable fromValues, int numberOfRows)
        {
            for (int i = 0; i < numberOfRows; i++)
            {
                CreateSamplePerson(fromValues, i);
            }

            fromValues.DataSet.AcceptChanges();
        }

        private void CreateSamplePerson(PersonDataSet.PersonDataTable fromValues, int id)
        {
            var temp = fromValues.NewPersonRow();

            temp.Id = id;
            temp.FirstName = UnitTestUtility.GetUniqueString();
            temp.LastName = UnitTestUtility.GetUniqueString();
            temp.EmailAddress = UnitTestUtility.GetUniqueString();

            fromValues.AddPersonRow(temp);
        }

        private void AssertAreEqual(
            PersonDataSet.PersonRow expected, IPerson actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<string>(expected.FirstName,
                actual.FirstName, "FirstName");
            Assert.AreEqual<string>(expected.LastName,
                actual.LastName, "LastName");
            Assert.AreEqual<string>(expected.EmailAddress,
                actual.EmailAddress, "EmailAddress");
        }
    }
}
